local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SliceCommander")
local LSM = LibStub("LibSharedMedia-3.0")
local _G = getfenv(0)

function SliceCommander:getOrder()
	_G['SC'].order = _G['SC'].order+1

	return _G['SC'].order
end

function SliceCommander:ArrayPos()
	local returnArray = {}
	local selected = {}
	local cd
	
	local count = 0
	for k, v in pairs(_G['SC'].SET.Pos) do
		if _G['SC'].BARS[string.upper(k)] == nil or _G['SC'].BARS[string.upper(k)]['name'] == nil then
			_G['SC'].SET.Pos[string.upper(k)] = nil--Remove spell
			_G['SC'].BARS[string.upper(k)] = nil
		else
			if string.sub(string.upper(k), -2) == 'CD' then
				cd = ' CD'
			else
				cd = ''
			end
			if not selected[v] then
					selected[v] = _G['SC'].BARS[string.upper(k)]['name'] .. cd
				
			else
				selected[v] = selected[v] .. ', ' .. _G['SC'].BARS[string.upper(k)]['name'] .. cd
			end
			count = count+1
		end
	end
	
	_G['SC'].countBars = count
	
	
	for key = -1*count, count, 1 do
		if key ~= 0 then
			if not selected[key] then
				returnArray[key] = key .. '. ' .. L['**Empty**']
			else
				returnArray[key] = key .. '. ' .. selected[key]
			end
		end
	end
	return returnArray
end

function SliceCommander:getColorCDDescription(spell)
	if string.sub(spell, -2) == 'CD' then
		return string.format(L["The color of %s CD bar"], SC_SPELL[spell])
	else
		return string.format(L["The color of %s bar"], SC_SPELL[spell])
	end
end

function SliceCommander:getColorSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = L["Color"],
		desc = SliceCommander:getColorCDDescription(spell),
		type = "color",
		set = function(info, r, g, b)
			_G['SC'].BARS[spell]['obj']:SetStatusBarColor(r, g, b)
			_G['SC'].BARS[spell]['color'] = {r, g, b}
		end,
		get = function(info) return unpack(_G['SC'].BARS[spell]['color']) end
	}
end

function SliceCommander:getCPColorSetting(type, id)
	return {
		order = SliceCommander:getOrder(),
		name = string.format(L["CP %s"], id),
		desc = L["CP color"],
		type = "color",
		set = function(info, r, g, b)
			_G['SC'].SET.CP[type][id] = {r, g, b }
			local maxCP = UnitPowerMax("player", SPELL_POWER_COMBO_POINTS)
			_G['SC'].BARS['CP']['obj'].combos[id].bg:SetVertexColor(unpack(_G['SC'].SET.CP[maxCP][id]))
		end,
		get = function(info) return unpack(_G['SC'].SET.CP[type][id]) end
	}
end

function SliceCommander:getPosSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = SliceCommander:getShowCDNamePosition(spell),
		desc = string.format(L["The place where to display the %s bar"], SC_SPELL[spell]),
		type = "select",
		width = "full",
		values = SliceCommander:ArrayPos(),
		set = function(info,val)
			_G['SC'].SET.Pos[spell] = val
		end,
		get = function(info) return _G['SC'].SET.Pos[spell] end
	}
end

function SliceCommander:getShowCDNamePosition(spell)
	if string.sub(spell, -2) == 'CD' then
		return L["CD Bar position"]
	else
		return L["Bar position"]
	end
end

function SliceCommander:getShowCDName(spell)
	if string.sub(spell, -2) == 'CD' then
		return L["Show CD bar"]
	else
		return L["Show bar"]
	end
end

function SliceCommander:getShowCDDescription(spell)
	if string.sub(spell, -2) == 'CD' then
		return string.format(L["Display the %s cooldown bar"], SC_SPELL[spell])
	else
		return string.format(L["Display the %s timer bar"], SC_SPELL[spell])
	end
end

function SliceCommander:getShowSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = SliceCommander:getShowCDName(spell),
		desc = SliceCommander:getShowCDDescription(spell),
		type = "toggle",
		set = function(info,val)
			_G['SC'].SET.display[spell] = val
		end,
		get = function(info) return _G['SC'].SET.display[spell] end
	}
end

function SliceCommander:getFlashSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = L["Flash when ready"],
		desc = L["The bar will keep flashing when it should be use."],
		type = "toggle",
		set = function(info,val)
			_G['SC'].SET.flash[spell] = val
		end,
		get = function(info) return _G['SC'].SET.flash[spell] end
	}
end

function SliceCommander:getHeaderSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = string.format(L["%s Settings"], SC_SPELL[spell]),
		type = "header"
	}
end

function SliceCommander:getDisplayDescriptionSetting()
	return {
		order = SliceCommander:getOrder(),
		name = L["displayDescription"],
		fontSize = "medium",
		type = "description"
	}
end

function SliceCommander:getSpellSettings(spell)
	local args = {
		['header'..spell] = {
			type = "header",
			order = SliceCommander:getOrder(),
			name = string.format(L["%s spell"], SC_SPELL[spell]),
		},
		['Show'..spell] = SliceCommander:getShowSetting(spell),
		['Color'..spell] = SliceCommander:getColorSetting(spell),
		['Pos'..spell] = SliceCommander:getPosSetting(spell),
		['Flash'..spell] = SliceCommander:getFlashSetting(spell)
	}
	if _G['SC'].SET.SpellSound ~= nil and _G['SC'].SET.SpellSound then
		args.SpellSound = SliceCommander:getSpellSound(spell)
	end
	return {
		name = SC_SPELL[spell],
		type = "group",
		order = SliceCommander:getOrder(),
		args = args
	}
end

function SliceCommander:getCDSettings(spell)
	local args = {
		['headerCD'..spell] = {
			type = "header",
			order = SliceCommander:getOrder(),
			name = string.format(L["%s CD"], SC_SPELL[spell]),
		},
		['Show'..spell] = SliceCommander:getShowSetting(spell),
		['Color'..spell] = SliceCommander:getColorSetting(spell),
		['Pos'..spell] = SliceCommander:getPosSetting(spell),
		['CDFlash'..spell] = SliceCommander:getFlashSetting(spell)
	}
	if _G['SC'].SET.SpellSound ~= nil and _G['SC'].SET.SpellSound then
		args.CDSpellSound = SliceCommander:getCDSpellSound(spell)
	end
	return {
		name = SC_SPELL[spell],
		type = "group",
		order = SliceCommander:getOrder(),
		args = args
	}
end

function SliceCommander:getRBSetting(spell)
	return {
		order = SliceCommander:getOrder(),
		name = SC_SPELL[spell],
		desc = string.format(L["%s text."], SC_SPELL[spell]),
		type = "input",
		width = "half",
		set = function(info,val) _G['SC'].SET.RB[spell] = val end,
		get = function(info) return _G['SC'].SET.RB[spell] end
	}
end
	
function SliceCommander:getSpellCDSettings(spell)
	local args = {
		['header'..spell] = {
			type = "header",
			order = SliceCommander:getOrder(),
			name = string.format(L["%s spell"], SC_SPELL[spell]),
		},
		['Show'..spell] = SliceCommander:getShowSetting(spell),
		['Color'..spell] = SliceCommander:getColorSetting(spell),
		['Pos'..spell] = SliceCommander:getPosSetting(spell),
		['Flash'..spell] = SliceCommander:getFlashSetting(spell),
		['headerCD'..spell] = {
			type = "header",
			order = SliceCommander:getOrder(),
			name = string.format(L["%s CD"], SC_SPELL[spell]),
		},
		['CDShow'..spell] = SliceCommander:getShowSetting(spell..'CD'),
		['CDColor'..spell] = SliceCommander:getColorSetting(spell..'CD'),
		['CDPos'..spell] = SliceCommander:getPosSetting(spell..'CD'),
		['CDFlash'..spell] = SliceCommander:getFlashSetting(spell..'CD')
	}
	if _G['SC'].SET.SpellSound ~= nil and _G['SC'].SET.SpellSound then
		args.SpellSound = SliceCommander:getSpellSound(spell)
		args.CDSpellSound = SliceCommander:getCDSpellSound(spell..'CD')
	end
	return {
		name = SC_SPELL[spell],
		type = "group",
		order = SliceCommander:getOrder(),
		args = args
	}
end

function SliceCommander:getCDSpellSound(spell)
	return {
		name = L["CD Sound"],
		type = "group",
		order = SliceCommander:getOrder(),
		args = {
			available = SliceCommander:getSoundSetting(L["On cooldown"], L["Alert when the spell is ready to be used"], spell, 'available')
		}
	}
end

function SliceCommander:getSpellSound(spell)
	return {
		name = L["Sound"],
		type = "group",
		order = SliceCommander:getOrder(),
		args = {
			Tick3 = SliceCommander:getSoundSetting(L["Alert - 3 sec"], L["Alert 3 secondes before fade"], spell, 'alert3'),
			Tick2 = SliceCommander:getSoundSetting(L["Alert - 2 sec"], L["Alert 2 secondes before fade"], spell, 'alert2'),
			Tick1 = SliceCommander:getSoundSetting(L["Alert - 1 sec"], L["Alert 1 secondes before fade"], spell, 'alert1'),
			Apply3 = SliceCommander:getSoundSetting(L["Refresh - 3 sec"], L["Refresh 3 secondes before fade"], spell, 'refresh3'),
			Apply2 = SliceCommander:getSoundSetting(L["Refresh - 2 sec"], L["Refresh 2 secondes before fade"], spell, 'refresh2'),
			Apply1 = SliceCommander:getSoundSetting(L["Refresh - 1 sec"], L["Refresh 1 secondes before fade"], spell, 'refresh1'),
			Applied = SliceCommander:getSoundSetting(L["Applied"], string.format(L["%s is applied"], SC_SPELL[spell]), spell, 'applied'),
			Expire = SliceCommander:getSoundSetting(L["Expired"], string.format(L["%s buff expire"], SC_SPELL[spell]), spell, 'expired')
		}
	}
end

function SliceCommander:getIconSpellSound(spell)
	return {
		Tick3 = SliceCommander:getIconSoundSetting(L["Alert - 3 sec"], L["Alert 3 secondes before fade"], spell, 'alert3'),
		Tick2 = SliceCommander:getIconSoundSetting(L["Alert - 2 sec"], L["Alert 2 secondes before fade"], spell, 'alert2'),
		Tick1 = SliceCommander:getIconSoundSetting(L["Alert - 1 sec"], L["Alert 1 secondes before fade"], spell, 'alert1'),
		Apply3 = SliceCommander:getIconSoundSetting(L["Refresh - 3 sec"], L["Refresh 3 secondes before fade"], spell, 'refresh3'),
		Apply2 = SliceCommander:getIconSoundSetting(L["Refresh - 2 sec"], L["Refresh 2 secondes before fade"], spell, 'refresh2'),
		Apply1 = SliceCommander:getIconSoundSetting(L["Refresh - 1 sec"], L["Refresh 1 secondes before fade"], spell, 'refresh1'),
		Applied = SliceCommander:getIconSoundSetting(L["Applied"], string.format(L["%s is applied"], SC_SPELL[spell]), spell, 'applied'),
		Expire = SliceCommander:getIconSoundSetting(L["Expired"], string.format(L["%s buff expire"], SC_SPELL[spell]), spell, 'expired')
	}
end

function SliceCommander:getEnergyMarkerSetting(spec, id)
	local maxi = 150
	if spec == 1 then
		maxi = 170
	end
	return {
		order = SliceCommander:getOrder(),
		name = string.format(L["Energy Marker %s"],id),
		type = "range",
		min = 0,
		max = maxi,
		step = 1,
		bigStep = 1,
		set = function(info,val)
			_G['SC'].SET.Energy[spec][id] = val
			SliceCommander:SetSparks()
		end,
		get = function(info) return _G['SC'].SET.Energy[spec][id] end,
		isPercent = false
	}
end

function SliceCommander:getEnergyMarkerSettings(spec)
	return {
		order = SliceCommander:getOrder(),
		name = L["Energy bar Settings"],
		type = "group",
		args = {
			Energy1 = SliceCommander:getEnergyMarkerSetting(spec, 1),
			Energy2 = SliceCommander:getEnergyMarkerSetting(spec, 2),
			Energy3 = SliceCommander:getEnergyMarkerSetting(spec, 3),
		}
	}
end

function SliceCommander:getSoundSetting(name, description, spell, id)
	return {
		order = SliceCommander:getOrder(),
		name = name,
		desc = description,
		type = "select",
		dialogControl = 'LSM30_Sound',
		values = LSM:HashTable("sound"),
		set = function(info,val)
			_G['SC'].BARS[spell]['sound'][id] = val
		end,
		get = function(info)
			if _G['SC'].BARS[spell]['sound'] == nil then _G['SC'].BARS[spell]['sound'] = {} end
			if _G['SC'].BARS[spell]['sound'][id] == nil then _G['SC'].BARS[spell]['sound'][id] = 'None' end
			return _G['SC'].BARS[spell]['sound'][id]
		end
	}
end

function SliceCommander:getIconSoundSetting(name, description, spell, id)
	return {
		order = SliceCommander:getOrder(),
		name = name,
		desc = description,
		type = "select",
		dialogControl = 'LSM30_Sound',
		values = LSM:HashTable("sound"),
		set = function(info,val)
			_G['SC'].ICONS[spell]['sound'][id] = val
		end,
		get = function(info)
			if _G['SC'].ICONS[spell]['sound'] == nil then _G['SC'].ICONS[spell]['sound'] = {} end
			if _G['SC'].ICONS[spell]['sound'][id] == nil then _G['SC'].ICONS[spell]['sound'][id] = 'None' end
			return _G['SC'].ICONS[spell]['sound'][id]
		end
	}
end

function SliceCommander:getShortcutSetting(sc_categ, sub_categ, spell, label)
	if label == nill then
		label = SC_SPELL[spell]
	end
	return {
		order = SliceCommander:getOrder(),
		type = "execute",
		name = label,
		func = function() AceConfigDialog:SelectGroup(sc_categ, sub_categ, spell) end,
	}
end

function SliceCommander:getShortcutHeader()
	return {
		order = SliceCommander:getOrder(),
		name = L["Shortcut"],
		type = "header"
	}
end

function SliceCommander:InitTable()
	local SC_OptionsTable = {}

	SC_OptionsTable["Sound"] = {
		name = L["Sound settings"],
		handler = SliceCommander,
		type = "group",
		childGroups = "tab",
		args = {
			SoundCombat = {
				order = SliceCommander:getOrder(),
				name = L["Sound Combat"],
				desc = L["Disable sounds out of combat"],
				type = "toggle",
				set = function(info,val) _G['SC'].SET.SoundCombat = val end,
				get = function(info) return _G['SC'].SET.SoundCombat end
			},
			SoundChannel = {
				order = SliceCommander:getOrder(),
				name = L["Sound channel"],
				desc = L["Choose the sound channel with which you want to play SliceCommander sounds. 'MASTER' must be use if you want to ear sounds when sound is disable. Other channels should be use if you want o lower volume."],
				type = "select",
				values = function()
					local returnArray = {
						["Master"] = "Master",
						["SFX"] = "SFX",
						["Ambience"] = "Ambience",
						["Music"] = "Music",
					}
					return returnArray
				end,
				set = function(info,val)
					_G['SC'].SET.soundChannel = val
				end,
				get = function(info) return _G['SC'].SET.soundChannel end
			},
			SoundVolume = {
				order = SliceCommander:getOrder(),
				name = L["Sound Volume"],
				desc = L["Warning: It will change the sound channel volume. If other sounds are played with this channel it will also change their volume."],
				type = "range",
				min = 0,
				max = 1,
				step = 0.05,
				bigStep = 0.1,
				set = function(info,val)
					SliceCommander:ChangeVolume(val)
				end,
				get = function(info) return tonumber(SliceCommander:GetVolume()) end,
				isPercent = true
			},
			SoundSpell = {
				order = SliceCommander:getOrder(),
				name = L["Enable spell sound notitication"],
				desc = L["Each spell will have full sound personalization in their respective settings."],
				type = "toggle",
				set = function(info,val)
					_G['SC'].SET.SpellSound = val
					SliceCommander:StaticPopup_Show('CONFIG_RL')
				end,
				get = function(info)
					if _G['SC'].SET.SpellSound == nil then
						_G['SC'].SET.SpellSound = false
					end
					return _G['SC'].SET.SpellSound
				end
			},
			ForceCDSpellSound = {
				order = SliceCommander:getOrder(),
				name = L["Force cooldown sound out of combat"],
				desc = L["Force the cooldown sound to be play even if you are not in combat."],
				type = "toggle",
				set = function(info,val)
					_G['SC'].SET.ForceCDSpellSound = val
				end,
				get = function(info)
					if _G['SC'].SET.ForceCDSpellSound == nil then
						_G['SC'].SET.ForceCDSpellSound = false
					end
					return _G['SC'].SET.ForceCDSpellSound
				end
			},
		}
	}

	SC_OptionsTable["General"] = {
		name = L["General settings"],
		handler = SliceCommander,
		type = "group",
		args = {--Display setting
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			General = {
				order = SliceCommander:getOrder(),
				name = L["General settings"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_CP = SliceCommander:getShortcutSetting("SliceCommander_General", "General", "CP"),
					Shortcut_HEALTH = SliceCommander:getShortcutSetting("SliceCommander_General", "General", "HEALTH"),
					Shortcut_THEALTH = SliceCommander:getShortcutSetting("SliceCommander_General", "General", "THEALTH"),
					Shortcut_THREAT = SliceCommander:getShortcutSetting("SliceCommander_General", "General", "THREAT"),
					Shortcut_POISON = {
						order = SliceCommander:getOrder(),
						type = "execute",
						name = string.format(L["%s Settings"], L["Poisons"]),
						func = function() AceConfigDialog:SelectGroup("SliceCommander_General", "General", "POISON") end,
					},
					CP = {
						order = SliceCommander:getOrder(),
						name = SC_SPELL['CP'],
						type = "group",
						args = {
							Bar = SliceCommander:getShowSetting('CP'),
							Pos = SliceCommander:getPosSetting('CP'),
							CP5 = {
								order = SliceCommander:getOrder(),
								name = "5 " .. string.format(L["%s Settings"], L["Combo Points"]),
								type = "header"
							},
							CP51 = SliceCommander:getCPColorSetting(5, 1),
							CP52 = SliceCommander:getCPColorSetting(5, 2),
							CP53 = SliceCommander:getCPColorSetting(5, 3),
							CP54 = SliceCommander:getCPColorSetting(5, 4),
							CP55 = SliceCommander:getCPColorSetting(5, 5),
							CP6 = {
								order = SliceCommander:getOrder(),
								name = "6 " .. string.format(L["%s Settings"], L["Combo Points"]),
								type = "header"
							},
							CP61 = SliceCommander:getCPColorSetting(6, 1),
							CP62 = SliceCommander:getCPColorSetting(6, 2),
							CP63 = SliceCommander:getCPColorSetting(6, 3),
							CP64 = SliceCommander:getCPColorSetting(6, 4),
							CP65 = SliceCommander:getCPColorSetting(6, 5),
							CP66 = SliceCommander:getCPColorSetting(6, 6),
							CP7 = {
								order = SliceCommander:getOrder(),
								name = "7 " .. string.format(L["%s Settings"], L["Combo Points"]),
								type = "header"
							},
							CP71 = SliceCommander:getCPColorSetting(7, 1),
							CP72 = SliceCommander:getCPColorSetting(7, 2),
							CP73 = SliceCommander:getCPColorSetting(7, 3),
							CP74 = SliceCommander:getCPColorSetting(7, 4),
							CP75 = SliceCommander:getCPColorSetting(7, 5),
							CP76 = SliceCommander:getCPColorSetting(7, 6),
							CP77 = SliceCommander:getCPColorSetting(7, 7),
						}
					},
					HEALTH = {
						order = SliceCommander:getOrder(),
						name = SC_SPELL['HEALTH'],
						type = "group",
						args = {
							Bar = SliceCommander:getShowSetting('HEALTH'),
							Pos = SliceCommander:getPosSetting('HEALTH'),
							healthPercentage = {
								order = SliceCommander:getOrder(),
								name = L["Show HP Value"],
								desc = L["Show HP value rather than health percentage."],
								type = "toggle",
								set = function(info,val)
									_G['SC'].SET.HEALTH.value = val
									SliceCommander:HealthBar('HEALTH')
								end,
								get = function(info) return _G['SC'].SET.HEALTH.value end
							},
							healthCombat = {
								order = SliceCommander:getOrder(),
								name = L["Hide out of combat"],
								desc = L["Hide health bar when you are not in combat."],
								type = "toggle",
								set = function(info,val)
									_G['SC'].SET.HEALTH.HideCombat = val
									SliceCommander:HealthBar('HEALTH')
								end,
								get = function(info) return _G['SC'].SET.HEALTH.HideCombat end
							},
							MidHealthUnder = {
								order = SliceCommander:getOrder(),
								name = L["Sound under 50% health"],
								desc = L["The sound play when your health reach 50% health"],
								type = "select",
								dialogControl = 'LSM30_Sound',
								values = LSM:HashTable("sound"),
								set = function(info,val)
									_G['SC'].SET.HEALTH.MidUnder = val
								end,
								get = function(info)
									return _G['SC'].SET.HEALTH.MidUnder
								end
							},
							LowHealthUnder = {
								order = SliceCommander:getOrder(),
								name = L["Sound under 35% health"],
								desc = L["The sound play when your health reach 35% health"],
								type = "select",
								dialogControl = 'LSM30_Sound',
								values = LSM:HashTable("sound"),
								set = function(info,val)
									_G['SC'].SET.HEALTH.LowUnder = val
								end,
								get = function(info)
									return _G['SC'].SET.HEALTH.LowUnder
								end
							},
						}
					},
					THEALTH = {
						order = SliceCommander:getOrder(),
						name = SC_SPELL['THEALTH'],
						type = "group",
						args = {
							Bar = SliceCommander:getShowSetting('THEALTH'),
							Pos = SliceCommander:getPosSetting('THEALTH'),
							healthPercentage = {
								order = SliceCommander:getOrder(),
								name = L["Show HP Value"],
								desc = L["Show HP value rather than health percentage."],
								type = "toggle",
								set = function(info,val)
									_G['SC'].SET.THEALTH.value = val
									SliceCommander:HealthBar('THEALTH')
								end,
								get = function(info) return _G['SC'].SET.THEALTH.value end
							},
							healthCombat = {
								order = SliceCommander:getOrder(),
								name = L["Hide out of combat"],
								desc = L["Hide health bar when you are not in combat."],
								type = "toggle",
								set = function(info,val)
									_G['SC'].SET.THEALTH.HideCombat = val
									SliceCommander:HealthBar('THEALTH')
								end,
								get = function(info) return _G['SC'].SET.THEALTH.HideCombat end
							},
						}
					},
					THREAT = {
						order = SliceCommander:getOrder(),
						name = SC_SPELL['THREAT'],
						type = "group",
						args = {
							Bar = SliceCommander:getShowSetting('THREAT'),
							Pos = SliceCommander:getPosSetting('THREAT'),
							ThreatGroupOnly = {--Show Threat bar only in group
								order = SliceCommander:getOrder(),
								name = L["Threat bar only in group"],
								desc = L["Display the Threat bar only if you are in a group or a raid"],
								type = "toggle",
								width = "full",
								set = function(info,val)
									_G['SC'].SET.ThreatGroupOnly = val
								end,
								get = function(info) return _G['SC'].SET.ThreatGroupOnly end
							},
						}
					},
					POISON = {
						name = string.format(L["%s Settings"], L["Poisons"]),
						type = "group",
						order = SliceCommander:getOrder(),
						args = {
							DPIconShow = {
								order = SliceCommander:getOrder(),
								name = L["Show poison icon"],
								desc = string.format(L["Display the %s icon when the timer left on your poison go under specified time. If you are in combat it only display the icon when it fade out."], SC_SPELL['DP']),
								type = "toggle",
								set = function(info,val)
									_G['SC'].SET.display.DPICON = val
								end,
								get = function(info) return _G['SC'].SET.display.DPICON end
							},
							DPTimeCheck = {
								order = SliceCommander:getOrder(),
								name = L["Minimal time"],
								desc = L["In minutes"],
								type = "range",
								min = 0,
								max = 59,
								step = 1,
								bigStep = 5,
								set = function(info,val)
									if strmatch(val,"%d") then
										_G['SC'].SET.display.DPTIME = val
									else
										_G['SC'].SET.display.DPTIME = 10
									end
								end,
								get = function(info) return _G['SC'].SET.display.DPTIME end
							},
							DPButtonEnable = {
								order = SliceCommander:getOrder(),
								name = L["Enable poison button"],
								desc = L["Enable poison button has no effect if poison icon is not shown. If it is enable there is two restrinction: You can't click through the area where the icon was when the icon is hide in combat and the button does not work when the icon is shown in combat."],
								type = "toggle",
								set = function(info,val)
									_G['SC'].SET.display.DPBUT = val
								end,
								get = function(info) return _G['SC'].SET.display.DPBUT end
							},
							DPSound = {
								order = SliceCommander:getOrder(),
								name = string.format(L["%s Sound"], SC_SPELL['DP']),
								desc = string.format(L["%s Sound. It has no effect if %s is not display."], SC_SPELL['DP'], SC_SPELL['DP']),
								type = "select",
								dialogControl = 'LSM30_Sound',
								values = LSM:HashTable("sound"),
								set = function(info,val)
									_G['SC'].SET.DPSound = val
								end,
								get = function(info) return _G['SC'].SET.DPSound end
							},
						}
					},
				}
			},
			Skills = {
				order = SliceCommander:getOrder(),
				name = L["Skills"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_SND = SliceCommander:getShortcutSetting("SliceCommander_General", "Skills", "SND"),
					Shortcut_CV = SliceCommander:getShortcutSetting("SliceCommander_General", "Skills", "CV"),
					Shortcut_KICD = SliceCommander:getShortcutSetting("SliceCommander_General", "Skills", "KICD"),
					Shortcut_VANCD = SliceCommander:getShortcutSetting("SliceCommander_General", "Skills", "VANCD"),
					Shortcut_KS = SliceCommander:getShortcutSetting("SliceCommander_General", "Skills", "KS"),
					Shortcut_CS = SliceCommander:getShortcutSetting("SliceCommander_General", "Skills", "CS"),
					Shortcut_SPR = SliceCommander:getShortcutSetting("SliceCommander_General", "Skills", "SPR"),
					SND = SliceCommander:getSpellSettings('SND'),
					CV = SliceCommander:getSpellCDSettings('CV'),
					KICD = {
						order = SliceCommander:getOrder(),
						name = SC_SPELL['KICD'],
						type = "group",
						args = {
							ShowKI = SliceCommander:getShowSetting('KICD'),
							ColorKI = SliceCommander:getColorSetting('KICD'),
							PosKI = SliceCommander:getPosSetting('KICD'),
							Flash = SliceCommander:getFlashSetting('KICD'),
							HeaderKINotification = {
								order = SliceCommander:getOrder(),
								name = L["Warning settings"],
								type = "header"
							},
							kickEnable = {
								order = SliceCommander:getOrder(),
								name = L["Enable kick warning"],
								desc = L["If enable, a warning message will be display when you succeed to interrupts spellcasting."],
								type = "toggle",
								set = function(info,val) _G['SC'].SET.kickEnable = val end,
								get = function(info) return _G['SC'].SET.kickEnable end
							},
							kickChannel = {
								order = SliceCommander:getOrder(),
								name = L["Channel"],
								desc = L["Choose which Channel the success message will be send to. Only one message is send. Then Channel priority order is Instance > Raid > Pary > Say."],
								type = "multiselect",
								values = {
									['INSTANCE_CHAT'] = L["Instance"],
									['RAID'] = L["Raid"],
									['PARTY'] = L["Party"],
									['SAY'] = L["Say"]
								},
								set = function(info, key, val) _G['SC'].SET.kickChannel[key] = val end,
								get = function(info, key) return _G['SC'].SET.kickChannel[key] end
							},
							kickMessage = {
								order = SliceCommander:getOrder(),
								name = L["Warning message"],
								desc = L["The warning message which will be display when you kick is successful."],
								type = "input",
								width = "full",
								set = function(info,val) _G['SC'].SET.kickMessage = val end,
								get = function(info) return _G['SC'].SET.kickMessage end
							},
							kickSpell = {
								order = SliceCommander:getOrder(),
								name = L["Add spell"],
								desc = L["Add kicked spell at the start of the message."],
								type = "toggle",
								set = function(info,val) _G['SC'].SET.kickSpell = val end,
								get = function(info) return _G['SC'].SET.kickSpell end
							},
						}
					},
					VANCD = SliceCommander:getCDSettings('VANCD'),
					KS = SliceCommander:getSpellCDSettings('KS'),
					CS = SliceCommander:getSpellSettings('CS'),
					SPR = SliceCommander:getSpellCDSettings('SPR'),
				}
			},
			Talents = {
				order = SliceCommander:getOrder(),
				name = L["Talents"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_CD = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "CDCD"),
					Shortcut_MD = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "MDCD"),
					Shortcut_AC = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "AC"),
					Shortcut_ER = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "ER"),
					Shortcut_SS = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "SS"),
					Shortcut_EVA = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "EVA"),
					Shortcut_COS = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "COS"),
					Shortcut_GG = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "GG"),
					Shortcut_SD = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents ", "SD"),
					Shortcut_FW = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "FW"),
					Shortcut_TOTCD = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "TOTCD"),
					Shortcut_FE = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "FE"),
					Shortcut_SUB = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "SUB"),
					Shortcut_SHICD = SliceCommander:getShortcutSetting("SliceCommander_General", "Skills", "SHICD"),
					Shortcut_COBCD = SliceCommander:getShortcutSetting("SliceCommander_General", "Skills", "COBCD"),
					Shortcut_TLT = SliceCommander:getShortcutSetting("SliceCommander_General", "Talents", "TLT"),
					CDCD = SliceCommander:getCDSettings('CDCD'),
					MDCD = SliceCommander:getCDSettings('MDCD'),
					AC = SliceCommander:getSpellSettings('AC'),
					ER = SliceCommander:getSpellCDSettings('ER'),
					SS = SliceCommander:getSpellCDSettings('SS'),
					EVA = SliceCommander:getSpellCDSettings('EVA'),
					COS = SliceCommander:getSpellCDSettings('COS'),
					GG = SliceCommander:getSpellCDSettings('GG'),
					SD = SliceCommander:getSpellCDSettings('SD'),
					FW = SliceCommander:getSpellSettings('FW'),
					FE = SliceCommander:getSpellCDSettings('FE'),
					SUB = SliceCommander:getSpellSettings('SUB'),
					TOTCD = {
						name = SC_SPELL['TOTCD'] .. ' CD',
						type = "group",
						order = SliceCommander:getOrder(),
						args = {
							headerCD = {
								type = "header",
								order = SliceCommander:getOrder(),
								name = L['Cooldown'],
							},
							TOTShow = SliceCommander:getShowSetting('TOTCD'),
							TOTColor = SliceCommander:getColorSetting('TOTCD'),
							TOTPos = SliceCommander:getPosSetting('TOTCD'),
							Flash = SliceCommander:getFlashSetting('TOTCD'),
							TOTTarget = {
								order = SliceCommander:getOrder(),
								name = L["Taget name"],
								desc = L["Write a macro which Tricks the target if it is friendly, if not the target of target. Except if a modifier is down the Tricks will target the player name specify here."],
								type = "input",
								set = function(info,val)
									_G['SC'].SET.TOTTarget = val
									SliceCommander:CreateTOTMacro()
								end,
								get = function(info) return _G['SC'].SET.TOTTarget end
							},
						}
					},
					SHICD = SliceCommander:getCDSettings('SHICD'),
					SHICD = SliceCommander:getCDSettings('COBCD'),
					TLT = SliceCommander:getSpellCDSettings('TLT'),
				}
			},
			TalentsPVP = {
				order = SliceCommander:getOrder(),
				name = L["PVP Talents"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_DFA = SliceCommander:getShortcutSetting("SliceCommander_General", "TalentsPVP", "DFACD"),
					DFACD = SliceCommander:getCDSettings('DFACD')
				}
			},
		}
	}

	SC_OptionsTable["Assassination"] = {
		name = L["Assassination settings"],
		handler = SliceCommander,
		type = "group",
		args = {--Display setting
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			General = {
				order = SliceCommander:getOrder(),
				name = L["General settings"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_EnergySetting = {
						order = SliceCommander:getOrder(),
						type = "execute",
						name = L["Energy bar Settings"],
						func = function() AceConfigDialog:SelectGroup("SliceCommander_Assassination", "General", "EnergySetting") end,
					},
					Shortcut_POISON = {
						order = SliceCommander:getOrder(),
						type = "execute",
						name = string.format(L["%s Settings"], L["Poisons"]),
						func = function() AceConfigDialog:SelectGroup("SliceCommander_Assassination", "General", "POISON") end,
					},
					EnergySetting = SliceCommander:getEnergyMarkerSettings(1),
					POISON = {
						name = string.format(L["%s Settings"], L["Poisons"]),
						type = "group",
						order = SliceCommander:getOrder(),
						args = {
							displayDescriptionChekcedSpec = {
								order = SliceCommander:getOrder(),
								name = L["displayDescriptionChekcedSpec"],
								fontSize = "medium",
								type = "description"
							},
							Lethal = {
								order = SliceCommander:getOrder(),
								name = L["Lethal poison"],
								type = "select",
								values = function()
									return { ['DPB'] = SC_SPELL['DPB'], ['WP'] = SC_SPELL['WP'] }
								end,
								set = function(info,val)
									local aSpec = GetSpecialization()
									if aSpec == 1 then
										_G['SC'].OTHER['POISON']['obj'].button:SetAttribute("spell1", SC_SPELL[val])
									end
									_G['SC'].SET.poison.mh[1] = val
								end,
								get = function(info)
									return _G['SC'].SET.poison.mh[1]
								end
							},
							Utility = {
								order = SliceCommander:getOrder(),
								name = L["Utility poison"],
								type = "select",
								values = function()
									return { ['CRP'] = SC_SPELL['CRP'], ['AKP'] = SC_SPELL['AKP'], ['ATP'] = SC_SPELL['ATP'] }
								end,
								set = function(info,val)
									local aSpec = GetSpecialization()
									if aSpec == 1 then
										_G['SC'].OTHER['POISON']['obj'].button:SetAttribute("spell2", SC_SPELL[val])
									end
									_G['SC'].SET.poison.oh[1] = val
								end,
								get = function(info)
									return _G['SC'].SET.poison.oh[1]
								end
							},
						}
					},
				}
			},
			Skills = {
				order = SliceCommander:getOrder(),
				name = L["Skills"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_RUP = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Skills", "RUP"),
					Shortcut_GAR = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Skills", "GAR"),
					Shortcut_ENV = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Skills", "ENV"),
					RUP = SliceCommander:getSpellSettings('RUP'),
					GAR = SliceCommander:getSpellCDSettings('GAR'),
					ENV = SliceCommander:getSpellSettings('ENV'),
				}
			},
			Talents = {
				order = SliceCommander:getOrder(),
				name = L["Talents"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_DP = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "DP"),
					Shortcut_EXCD = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "EXCD"),
					Shortcut_EP = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "EP"),
					Shortcut_BLS = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "BLS"),
					Shortcut_MA = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "MA"),
					Shortcut_CT = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "CT"),
					Shortcut_SP = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "SP"),
					Shortcut_SBS = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "SBS"),
					Shortcut_KB = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "KB"),
					Shortcut_DTM = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "DTM"),
					Shortcut_ICCD = SliceCommander:getShortcutSetting("SliceCommander_Assassination", "Talents", "ICCD"),
					DP = SliceCommander:getSpellSettings('DP'),
					EXCD = SliceCommander:getCDSettings('EXCD'),
					EP = SliceCommander:getSpellSettings('EP'),
					BLS = SliceCommander:getSpellSettings('BLS'),
					MA = SliceCommander:getSpellSettings('MA'),
					CT = SliceCommander:getSpellSettings('CT'),
					SP = SliceCommander:getSpellCDSettings('SP'),
					SBS = SliceCommander:getSpellCDSettings('SBS'),
					KB = SliceCommander:getSpellCDSettings('KB'),
					DTM = SliceCommander:getSpellCDSettings('DTM'),
					ICCD = SliceCommander:getCDSettings('ICCD'),
				}
			}
		}
	}

	SC_OptionsTable["Outlaw"] = {
		name = L["Outlaw settings"],
		handler = SliceCommander,
		type = "group",
		args = {
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			General = {
				order = SliceCommander:getOrder(),
				name = L["General settings"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_EnergySetting = {
						order = SliceCommander:getOrder(),
						type = "execute",
						name = L["Energy bar Settings"],
						func = function() AceConfigDialog:SelectGroup("SliceCommander_Outlaw", "General", "EnergySetting") end,
					},
					Shortcut_POISON = {
						order = SliceCommander:getOrder(),
						type = "execute",
						name = string.format(L["%s Settings"], L["Poisons"]),
						func = function() AceConfigDialog:SelectGroup("SliceCommander_Outlaw", "General", "POISON") end,
					},
					EnergySetting = SliceCommander:getEnergyMarkerSettings(2),
					POISON = {
						name = string.format(L["%s Settings"], L["Poisons"]),
						type = "group",
						order = SliceCommander:getOrder(),
						args = {
							displayDescriptionChekcedSpec = {
								order = SliceCommander:getOrder(),
								name = L["displayDescriptionChekcedSpec"],
								fontSize = "medium",
								type = "description"
							},
							Lethal = {
								order = SliceCommander:getOrder(),
								name = L["Lethal poison"],
								type = "select",
								values = function()
									return { ['IP'] = SC_SPELL['IP'], ['WP'] = SC_SPELL['WP'] }
								end,
								set = function(info,val)
									local aSpec = GetSpecialization()
									if aSpec == 2 then
										_G['SC'].OTHER['POISON']['obj'].button:SetAttribute("spell1", SC_SPELL[val])
									end
									_G['SC'].OTHER['POISON']['obj'].button:SetAttribute("spell1", SC_SPELL[val])
									_G['SC'].SET.poison.mh[2] = val
								end,
								get = function(info)
									return _G['SC'].SET.poison.mh[2]
								end
							},
							Utility = {
								order = SliceCommander:getOrder(),
								name = L["Utility poison"],
								type = "select",
								values = function()
									return { ['CRP'] = SC_SPELL['CRP'], ['AKP'] = SC_SPELL['AKP'], ['ATP'] = SC_SPELL['ATP'] }
								end,
								set = function(info,val)
									local aSpec = GetSpecialization()
									if aSpec == 2 then
										_G['SC'].OTHER['POISON']['obj'].button:SetAttribute("spell2", SC_SPELL[val])
									end
									_G['SC'].SET.poison.oh[2] = val
								end,
								get = function(info)
									return _G['SC'].SET.poison.oh[2]
								end
							},
						}
					},
				}
			},
			Skills = {
				order = SliceCommander:getOrder(),
				name = L["Skills"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_BF = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Skills", "BF"),
					Shortcut_BE = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Skills", "BE"),
					Shortcut_OP = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Skills", "OP"),
					BF = SliceCommander:getSpellCDSettings('BF'),
					BE = SliceCommander:getSpellCDSettings('BE'),
					OP = SliceCommander:getSpellSettings('OP'),
				}
			},
			Talents = {
				order = SliceCommander:getOrder(),
				name = L["Talents"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_GS = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "GS"),
					Shortcut_KSPCD = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "KSPCD"),
					Shortcut_LD = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "LD"),
					Shortcut_BRCD = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "BRCD"),
					Shortcut_CDB = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "CDB"),
					Shortcut_SP = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "SP"),
					Shortcut_GHCD = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "GHCD"),
					Shortcut_AR = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "AR"),
					Shortcut_RB = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "RB"),
					Shortcut_KIRCD = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "KIRCD"),
					Shortcut_AUD = SliceCommander:getShortcutSetting("SliceCommander_Outlaw", "Talents", "AUD"),
					EnergySetting = SliceCommander:getEnergyMarkerSettings(1),
					GS = SliceCommander:getSpellCDSettings('GS'),
					KSPCD = SliceCommander:getCDSettings('KSPCD'),
					LD = SliceCommander:getSpellSettings('LD'),
					BRCD = SliceCommander:getCDSettings('BRCD'),
					CDB = SliceCommander:getSpellCDSettings('CDB'),
					SP = SliceCommander:getSpellCDSettings('SP'),
					GHCD = SliceCommander:getCDSettings('GHCD'),
					AR = {
						order = SliceCommander:getOrder(),
						name = SC_SPELL['AR'],
						type = "group",
						args = {
							header = {
								type = "header",
								order = SliceCommander:getOrder(),
								name = L['Spell'],
							},
							ARShow = SliceCommander:getShowSetting('AR'),
							ARColor = SliceCommander:getColorSetting('AR'),
							ARPos = SliceCommander:getPosSetting('AR'),
							Flash = SliceCommander:getFlashSetting('AR'),
							ARMarioMod = {
								order = SliceCommander:getOrder(),
								name = L["Enable Mario mod"],
								desc = L["Enable mario star music during Adrenaline rush buff"],
								type = "toggle",
								set = function(info,val)
									if _G['SC'].BARS['AR']['sound'] == nil then _G['SC'].BARS['AR']['sound'] = {} end
									if val then
										SliceCommander:PlaySoundReal('Mario')
										_G['SC'].BARS['AR']['sound']['applied'] = 'Mario'
									else
										if _G['SC'].previousSound then
											StopSound(_G['SC'].previousSound)
										end
										_G['SC'].BARS['AR']['sound']['applied'] = 'None'
									end
									_G['SC'].SET.ARMarioMod = val
								end,
								get = function(info) return _G['SC'].SET.ARMarioMod end
							},
							headerCD = {
								type = "header",
								order = SliceCommander:getOrder(),
								name = L['Cooldown'],
							},
							ARCDShow = SliceCommander:getShowSetting('ARCD'),
							ARCDColor = SliceCommander:getColorSetting('ARCD'),
							ARCDPos = SliceCommander:getPosSetting('ARCD'),
							FlashCD = SliceCommander:getFlashSetting('ARCD')
						}
					},
					RB = {
						name = SC_SPELL['RB'],
						type = "group",
						order = SliceCommander:getOrder(),
						args = SliceCommander_TableMerge(
							SliceCommander:getSpellCDSettings('RB').args,
							{
								headerSpecial = {
									type = "header",
									order = SliceCommander:getOrder(),
									name = L['Special settings'],
								},
								replaceName = {
									order = SliceCommander:getOrder(),
									name = L["Replace spell name with each following spell text"],
									desc = L["Each active buff actually up will be displayed in the spell name."],
									type = "toggle",
									set = function(info,val)
										_G['SC'].SET.display.RBName = val
									end,
									get = function(info) return _G['SC'].SET.display.RBName end
								},
								SC = SliceCommander:getRBSetting('SC'),
								GM = SliceCommander:getRBSetting('GM'),
								RP = SliceCommander:getRBSetting('RP'),
								TB = SliceCommander:getRBSetting('TB'),
								BT = SliceCommander:getRBSetting('BT'),
								BS = SliceCommander:getRBSetting('BS')
							}
						)
					},
					KIRCD = SliceCommander:getCDSettings('KIRCD'),
					AUD = {
						name = SC_SPELL['AUD'],
						type = "group",
						order = SliceCommander:getOrder(),
						args = SliceCommander_TableMerge(
							{
								toggleAUD = {
									order = SliceCommander:getOrder(),
									name = L["Show Audacity icon"],
									desc = L["Show audacity icon when ambush utilisation without stealth is possible."],
									type = "toggle",
									set = function(info,val) _G['SC'].SET.display.AUD = val end,
									get = function(info) return _G['SC'].SET.display.AUD end
								},
								AUD_soundHeader = {
									order = SliceCommander:getOrder(),
									name = L["Sound settings"],
									type = "header"
								}
							},
							SliceCommander:getIconSpellSound('AUD')
						)
					},
				}
			}
		}
	}
	if _G['SC'].SET.SpellSound ~= nil and _G['SC'].SET.SpellSound then
		SC_OptionsTable["Outlaw"]['args']['Talents']['args']['AR']['args']['ARCDSpellSound'] = SliceCommander:getCDSpellSound('ARCD')
	end

	SC_OptionsTable["Sublety"] = {
		name = L["Subtlety settings"],
		handler = SliceCommander,
		type = "group",
		args = {--Display setting
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			General = {
				order = SliceCommander:getOrder(),
				name = L["General settings"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_EnergySetting = {
						order = SliceCommander:getOrder(),
						type = "execute",
						name = L["Energy bar Settings"],
						func = function() AceConfigDialog:SelectGroup("SliceCommander_Sublety", "General", "EnergySetting") end,
					},
					Shortcut_POISON = {
						order = SliceCommander:getOrder(),
						type = "execute",
						name = string.format(L["%s Settings"], L["Poisons"]),
						func = function() AceConfigDialog:SelectGroup("SliceCommander_Sublety", "General", "POISON") end,
					},
					EnergySetting = SliceCommander:getEnergyMarkerSettings(3),
					POISON = {
						name = string.format(L["%s Settings"], L["Poisons"]),
						type = "group",
						order = SliceCommander:getOrder(),
						args = {
							displayDescriptionChekcedSpec = {
								order = SliceCommander:getOrder(),
								name = L["displayDescriptionChekcedSpec"],
								fontSize = "medium",
								type = "description"
							},
							Lethal = {
								order = SliceCommander:getOrder(),
								name = L["Lethal poison"],
								type = "select",
								values = function()
									return { ['IP'] = SC_SPELL['IP'], ['WP'] = SC_SPELL['WP'] }
								end,
								set = function(info,val)
									local aSpec = GetSpecialization()
									if aSpec == 3 then
										_G['SC'].OTHER['POISON']['obj'].button:SetAttribute("spell1", SC_SPELL[val])
									end
									_G['SC'].SET.poison.mh[3] = val
								end,
								get = function(info)
									return _G['SC'].SET.poison.mh[3]
								end
							},
							Utility = {
								order = SliceCommander:getOrder(),
								name = L["Utility poison"],
								type = "select",
								values = function()
									return { ['CRP'] = SC_SPELL['CRP'], ['AKP'] = SC_SPELL['AKP'], ['ATP'] = SC_SPELL['ATP'] }
								end,
								set = function(info,val)
									local aSpec = GetSpecialization()
									if aSpec == 3 then
										_G['SC'].OTHER['POISON']['obj'].button:SetAttribute("spell2", SC_SPELL[val])
									end
									_G['SC'].SET.poison.oh[3] = val
								end,
								get = function(info)
									return _G['SC'].SET.poison.oh[3]
								end
							},
						}
					},
				}
			},
			Skills = {
				order = SliceCommander:getOrder(),
				name = L["Skills"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_RUP = SliceCommander:getShortcutSetting("SliceCommander_Sublety", "Skills", "RUP"),
					Shortcut_SD = SliceCommander:getShortcutSetting("SliceCommander_Sublety", "Skills", "SD"),
					Shortcut_SOD = SliceCommander:getShortcutSetting("SliceCommander_Sublety", "Skills", "SOD"),
					RUP = SliceCommander:getSpellSettings('RUP'),
					SD = SliceCommander:getSpellCDSettings('SD'),
					SOD = SliceCommander:getSpellCDSettings('SOD'),
				}
			},
			Talents = {
				order = SliceCommander:getOrder(),
				name = L["Talents"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_SHT = SliceCommander:getShortcutSetting("SliceCommander_Sublety", "Talents", "SHT"),
					Shortcut_STHCD = SliceCommander:getShortcutSetting("SliceCommander_Sublety", "Talents", "STHCD"),
					Shortcut_SP = SliceCommander:getShortcutSetting("SliceCommander_Sublety", "Talents", "SP"),
					Shortcut_FLA = SliceCommander:getShortcutSetting("SliceCommander_Sublety", "Talents", "FLA"),
					Shortcut_SB = SliceCommander:getShortcutSetting("SliceCommander_Sublety", "Talents", "SB"),
					SHT = SliceCommander:getSpellCDSettings('SHT'),
					STHCD = SliceCommander:getCDSettings('STHCD'),
					SP = SliceCommander:getSpellCDSettings('SP'),
					FLA = SliceCommander:getSpellCDSettings('FLA'),
					SB = SliceCommander:getSpellCDSettings('SB'),
				}
			}
		}
	}

	SC_OptionsTable["Complementary"] = {
		name = L["Old settings"],
		handler = SliceCommander,
		type = "group",
		args = {--Display setting
			displayDescription = SliceCommander:getDisplayDescriptionSetting(),
			General = {
				order = SliceCommander:getOrder(),
				name = L["General settings"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_MAI = SliceCommander:getShortcutSetting("SliceCommander_Complementary", "General", "MAI"),
					MAI = SliceCommander:getSpellSettings('MAI'),
				}
			},
			Assassination = {
				order = SliceCommander:getOrder(),
				name = L["Assassination settings"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_BA = SliceCommander:getShortcutSetting("SliceCommander_Complementary", "Assassination", "BA"),
					Shortcut_ST = SliceCommander:getShortcutSetting("SliceCommander_Complementary", "Assassination", "ST"),
					BA = SliceCommander:getSpellSettings('BA'),
					ST = SliceCommander:getSpellSettings('ST'),
				}
			},
			Outlaw = {
				order = SliceCommander:getOrder(),
				name = L["Outlaw settings"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_GSWW = SliceCommander:getShortcutSetting("SliceCommander_Complementary", "Outlaw", "GSWW"),
					GSWW = SliceCommander:getSpellSettings('GSWW'),
				}
			},
			Subtlety = {
				order = SliceCommander:getOrder(),
				name = L["Subtlety settings"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_GB = SliceCommander:getShortcutSetting("SliceCommander_Complementary", "Subtlety", "GB"),
					GB = SliceCommander:getSpellCDSettings('GB'),
				}
			},
			AzeriteEssence = {
				order = SliceCommander:getOrder(),
				name = L["Azerite Essence"],
				type = "group",
				childGroups = "tree",
				args = {
					Shortcut = SliceCommander:getShortcutHeader(),
					Shortcut_ESSECD = SliceCommander:getShortcutSetting("SliceCommander_Complementary", "AzeriteEssence", "ESSECD"),
					ESSECD = SliceCommander:getCDSettings('ESSECD')
				}
			}
		}
	}

	SC_OptionsTable["SliceCommander"] = {
		name = "SliceCommander",
		handler = SliceCommander,
		type = "group",
		args = {--SliceCommander
			IsLocked = {--Lock
				order = SliceCommander:getOrder(),
				name = L["Lock"],
				desc = L["Disable Left Click to Move"],
				type = "toggle",
				set = function(info,val)
					SliceCommander:LockCkecked(val)
				end,
				get = function(info) return _G['SC'].SET.IsLocked end
			},
			FullTimer = {--Full timer
				order = SliceCommander:getOrder(),
				name = L["Full Timer"],
				desc = L["Make timer full bar duration"],
				type = "toggle",
				set = function(info,val) _G['SC'].SET.FullTimer = val end,
				get = function(info) return _G['SC'].SET.FullTimer end
			},
			PadLatency = {--Latency
				order = SliceCommander:getOrder(),
				name = L["Enable Latency"],
				desc = L["Pad Alerts with Latency"],
				type = "toggle",
				set = function(info,val) _G['SC'].SET.PadLatency = val end,
				get = function(info) return _G['SC'].SET.PadLatency end
			},
			SpellText = {--Spell text
				order = SliceCommander:getOrder(),
				name = L["Spell Text"],
				desc = L["Enable spell text on bars"],
				type = "toggle",
				set = function(info,val) _G['SC'].SET.SpellText = val end,
				get = function(info) return _G['SC'].SET.SpellText end
			},
			GCD = {--Spell text
				order = SliceCommander:getOrder(),
				name = L["GCD Bar"],
				desc = L["Enable GCD bar"],
				type = "toggle",
				set = function(info,val) _G['SC'].SET.displayGCD = val end,
				get = function(info) return _G['SC'].SET.displayGCD end
			},
			PixelPerfect = {
				order = SliceCommander:getOrder(),
				name = L["Pixel Perfect"],
				desc = L["Should be enabled. If you change this option state you have to reload interface. This option is automatically disabled after a reload to only force uiscale one time."],
				type = "toggle",
				set = function(info,val)
					_G['SC'].SET.PixelPerfect = val
					SliceCommander:StaticPopup_Show('CONFIG_RL')
				end,
				get = function(info) return _G['SC'].SET.PixelPerfect end
			},
			headerDisplaySetting = {--CDisplay Setting
				order = SliceCommander:getOrder(),
				name = L["Display Setting"],
				type = "header"
			},
			IconSize = {--Icon size
				order = SliceCommander:getOrder(),
				name = L["Icon size"],
				desc = L["Icon size"],
				type = "range",
				min = 10,
				max = 50,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					_G['SC'].SET.IconSize = val
					SliceCommander:SetIconSize(val)
				end,
				get = function(info) return _G['SC'].SET.IconSize end,
				isPercent = false
			},
			Width = {--Width
				order = SliceCommander:getOrder(),
				name = L["Width"],
				desc = L["widthDescription"],
				type = "range",
				min = 125,
				max = 900,
				step = 1,
				bigStep = 5,
				set = function(info,val)
					_G['SC'].SET.Width = val
					if _G['SC'].SET.skin then
						SliceCommander:ApplySkin(false)
					end
					SliceCommander:SetWidth(val)
					if _G['SC'].SET.skin then
						SliceCommander:ApplySkin(true)
					end

				end,
				get = function(info) return _G['SC'].SET.Width end,
				isPercent = false
			},
			TimerHeight = {--Timer height
				order = SliceCommander:getOrder(),
				name = L["Timer Height"],
				desc = L["Timer Bar Height"],
				type = "range",
				min = 1,
				max = 50,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					_G['SC'].SET.TimerHeight = val
					SliceCommander:SetTimersHeight(val)
					SliceCommander:ApplySkin(_G['SC'].SET.skin)
				end,
				get = function(info) return _G['SC'].SET.TimerHeight end,
				isPercent = false
			},
			BarMargin = {--Margin
				order = SliceCommander:getOrder(),
				name = L["Bar Margin"],
				desc = L["Bar Margin"],
				type = "range",
				min = -2,
				max = 10,
				step = 1,
				bigStep = 1,
				set = function(info,val) _G['SC'].SET.BarMargin = val end,
				get = function(info) return _G['SC'].SET.BarMargin end,
				isPercent = false
			},
			FrameOpacity = {--Opacity
				order = SliceCommander:getOrder(),
				name = L["Frame opacity"],
				desc = L["Frame opacity"],
				type = "range",
				min = 25,
				max = 100,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					SliceCommander:FrameOpacity(val)
					_G['SC'].SET.FrameOpacity = val
				end,
				get = function(info) return _G['SC'].SET.FrameOpacity end,
				isPercent = false
			},
			BarTexture = {--Texture
				order = SliceCommander:getOrder(),
				name = L["Bar Texture"],
				desc = L["Bar Texture for timer and energy"],
				type = "select",
				dialogControl = 'LSM30_Statusbar',
				values = LSM:HashTable("statusbar"),
				set = function(info,val)
					_G['SC'].SET.BarTexture = val
					SliceCommander:Config_RetextureBars(SliceCommander:BarTexture())
				end,
				get = function(info) return _G['SC'].SET.BarTexture end
			},
			Font = {
				order = SliceCommander:getOrder(),
				name = L["Font"],
				type = "select",
				dialogControl = 'LSM30_Font',
				values = LSM:HashTable("font"),
				set = function(info,val)
					_G['SC'].SET.font = val
					SliceCommander:StaticPopup_Show('CONFIG_RL')
				end,
				get = function(info) return _G['SC'].SET.font end
			},
			headerEnergySetting = {
				order = SliceCommander:getOrder(),
				name = L["Energy bar Settings"],
				type = "header"
			},
			HideEnergy = {
				order = SliceCommander:getOrder(),
				name = L["Hide Energy bar"],
				desc = L["Hide Energy bar"],
				type = "toggle",
				set = function(info,val)
					if val then
						VTimerEnergy:Hide()
					else
						VTimerEnergy:Show()
					end
					_G['SC'].SET.HideEnergy = val
				end,
				get = function(info) return _G['SC'].SET.HideEnergy end
			},
			EnergyHeight = {
				order = SliceCommander:getOrder(),
				name = L["Energy Height"],
				desc = L["Energy Bar Height"],
				type = "range",
				min = 1,
				max = 40,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					_G['SC'].SET.EnergyHeight = val
					SliceCommander:SetEnergyHeight(val)
				end,
				get = function(info) return _G['SC'].SET.EnergyHeight end,
				isPercent = false
			},
			EnergyTrans = {
				order = SliceCommander:getOrder(),
				name = L["Transparency (Full)"],
				desc = L["The fade out value of the energy bar when it is full."],
				type = "range",
				min = 0,
				max = 100,
				step = 1,
				bigStep = 1,
				set = function(info,val)
					SliceCommander:SetTransparency(val)
					_G['SC'].SET.EnergyTrans = val
				end,
				get = function(info) return _G['SC'].SET.EnergyTrans end,
				isPercent = false
			},
			EnergyColor = {
				order = SliceCommander:getOrder(),
				name = L["Color"],
				type = "color",
				set = function(info, r, g, b)
					VTimerEnergy:SetStatusBarColor(r, g, b)
					_G['SC'].SET.energyColor = {r, g, b}
				end,
				get = function(info) return unpack(_G['SC'].SET.energyColor) end
			},
			headerTicksSetting = {
				order = SliceCommander:getOrder(),
				name = L["Ticks"],
				type = "header"
			},
			PandemicToggle = {
				order = SliceCommander:getOrder(),
				name = L["Enable pandemic tick"],
				desc = L["A small red bar will be displaid on each timer bar impacted by the mechanic to show you when you can refresh the corresponding buff/debuff."],
				type = "toggle",
				width = "full",
				set = function(info,val)
					_G['SC'].SET.Pandemic = val
				end,
				get = function(info) return _G['SC'].SET.Pandemic end
			},
			TickToggle = {
				order = SliceCommander:getOrder(),
				name = L["Enable Dot/Hot ticks indicator"],
				desc = L["A small white bar will be displaid on each HoT / DoT timer to show you when the effect really occur."],
				type = "toggle",
				width = "full",
				set = function(info,val)
					_G['SC'].SET.Tick = val
				end,
				get = function(info) return _G['SC'].SET.Tick end
			},
		}
	}

	return SC_OptionsTable
end

function SliceCommander:ResetOptions()
	SliceCommander:OnDisable()
	SliceCommanderFrame:SetScript("OnUpdate", null)
	_G['SC'] = { PopupDialogs = null }
	SliceCommander:StaticPopup_Show('CONFIG_RL')
end

function SliceCommander:InitConfigurationTable()
	local config_table = SliceCommander:InitTable()

	AceConfig:RegisterOptionsTable("SliceCommander", config_table.SliceCommander)
	AceConfig:RegisterOptionsTable("SliceCommander_General", config_table.General)
	AceConfig:RegisterOptionsTable("SliceCommander_Assassination", config_table.Assassination)
	AceConfig:RegisterOptionsTable("SliceCommander_Outlaw", config_table.Outlaw)
	AceConfig:RegisterOptionsTable("SliceCommander_Sublety", config_table.Sublety)
	AceConfig:RegisterOptionsTable("SliceCommander_Complementary", config_table.Complementary)
	AceConfig:RegisterOptionsTable("SliceCommander_Sound", config_table.Sound)

	local SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander","SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_General",L["General settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Assassination",L["Assassination settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Outlaw",L["Outlaw settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Sublety",L["Subtlety settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Complementary",L["Old settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end

	SetDefaultOpttion = AceConfigDialog:AddToBlizOptions("SliceCommander_Sound",L["Sound settings"],"SliceCommander "..GetAddOnMetadata("SliceCommander", "Version"))
	SetDefaultOpttion.default = function() SliceCommander:ResetOptions() end
end